=== modified file 'NEWS'
--- NEWS	2006-06-16 23:50:37 +0000
+++ NEWS	2006-06-17 01:13:13 +0000
@@ -87,8 +87,8 @@
       that do not use '.bzr' to store their data - i.e. '.svn', '.hg' etc.
       (Robert Collins, Jelmer Vernooij).
 
-    * bzrlib.diff.external_diff can be redirected to any filelike object that
-      has a file descriptor. Use subprocess instead of spawnvp.
+    * bzrlib.diff.external_diff can be redirected to any file-like object.
+      Uses subprocess instead of spawnvp.
       (#4047, #48914, James Henstridge, John Arbash Meinel)
 
 bzr 0.8.2  2006-05-17

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2006-06-17 01:04:57 +0000
+++ bzrlib/diff.py	2006-06-17 01:13:13 +0000
@@ -86,9 +86,12 @@
 def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
-    if not hasattr(to_file, 'fileno'):
-        raise NotImplementedError("sorry, can't send external diff other "
-                                  "than to a file descriptor", to_file)
+    if hasattr(to_file, 'fileno'):
+        out_file = to_file
+        have_fileno = True
+    else:
+        out_file = subprocess.PIPE
+        have_fileno = False
     
     # make sure our own output is properly ordered before the diff
     to_file.flush()
@@ -149,12 +152,15 @@
         try:
             pipe = subprocess.Popen(diffcmd,
                                     stdin=subprocess.PIPE,
-                                    stdout=to_file)
+                                    stdout=out_file)
         except OSError, e:
             if e.errno == errno.ENOENT:
                 raise errors.NoDiff(str(e))
             raise
         pipe.stdin.close()
+
+        if not have_fileno:
+            to_file.write(pipe.stdout.read())
         rc = pipe.wait()
         
         if rc != 0 and rc != 1:

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2006-06-17 01:04:57 +0000
+++ bzrlib/tests/test_diff.py	2006-06-17 01:13:13 +0000
@@ -33,8 +33,12 @@
     return output.readlines()
 
 
-def external_udiff_lines(old, new):
-    output = TemporaryFile()
+def external_udiff_lines(old, new, use_stringio=False):
+    if use_stringio:
+        # StringIO has no fileno, so it tests a different codepath
+        output = StringIO()
+    else:
+        output = TemporaryFile()
     try:
         external_diff('old', old, 'new', new, output, diff_opts=['-u'])
     except errors.NoDiff:
@@ -95,6 +99,14 @@
     def test_external_diff(self):
         lines = external_udiff_lines(['boo\n'], ['goo\n'])
         self.check_patch(lines)
+
+    def test_external_diff_no_fileno(self):
+        # Make sure that we can handle not having a fileno, even
+        # if the diff is large
+        lines = external_udiff_lines(['boo\n']*10000,
+                                     ['goo\n']*10000,
+                                     use_stringio=True)
+        self.check_patch(lines)
         
     def test_internal_diff_default(self):
         # Default internal diff encoding is utf8

